namespace eval ImportImg2Mat {
    variable cwd [file normalize [file dirname [info script]]]

    proc amIMaster { } {
	if {[info exists ::argv0] &&
	    [file dirname [file normalize [info script]/...]] eq
	    [file dirname [file normalize $::argv0/...]]} {
	    return 1
	} else {
	    return 0
	}
    }
    if {[amIMaster]} {
	lappend ::auto_path [file join $cwd lib]
    }
}

package require Tk
package require ClipImage
package require Img
package require BWidget
package require msgcat

namespace import msgcat::*

namespace eval ImportImg2Mat {
    variable win
    variable options
    variable inputFile
    variable inputSerie
    variable checkROI 0
    variable data
    variable widgets
    variable images
    
    if { [ lsearch [ image names ] folder_yellow_16 ] == -1 } {
	image create photo folder_yellow_16 -data {
	    R0lGODlhEAAQAPYAAK+KGLGMGLOOGbKNGrKMG7OMG7OOG7SOGbSPGbWPH7SQGaOFI8GLNMmJN8Se
	    MMSXOcSfO9eQOuOePN6xLcOhN8WgNMahNcehNcmjNMijN8mmNcukN8ulN8umNsumN8KgOMeiOMei
	    OcemPMekPselPsemPtGnN9uqNeG9K+K4LeO/LOS8LeW/LeytO/G5OeTALPTHN/XMNvTBOffQNcun
	    QtGtQtCuSd6+Q922Sum8S/TCTvjETvjITvjOTe/USO7USfrTTfzaS/veTPzgTPzkTfzmUfvpXd3A
	    ZurTg/rrhvLgl/nrqAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5
	    BAEAAEwALAAAAAAQABAAAAeSgEyCg4SFhoeGBwgICgGIggYTKSssAgqXlwiEAC4yMDEzKi8vKioo
	    A4MALScYGxwesLAdFRcgTAASJj9GSUu+v0tKFkwEERlCREW8wEtIIUwFDTVAQUPJy75HggUMODw9
	    1NbKvTbbDzk7O9/h1w7bEDrp6evVPhqCCTTy++s3gwsfSJQoQWIEiYIjRFB4xLAhoUAAOw==
	}
    }

    proc _puts {args} {
	if {[info command .gid.central] ne ""} {
	    WarnWinText {*}$args
	} else {
	    puts {*}$args
	}
    }
    
    proc GetTmpDir {} {
	global env tcl_platform
	
	if {$tcl_platform(platform) eq "unix"} {
	    set TmpDir /tmp
	} else {
	    if {[info exist env(TEMP)] && [file exists $env(TEMP)]} {
		set TmpDir $env(TEMP)
	    } elseif {[info exist env(TMP)] && [file exists $env(TMP)]} {
		set TmpDir $env(TMP)
	    } else {
		set TmpDir "C:/"
	    }
	}
	return $TmpDir
    }
    
    proc OnCheckROI { } {
	variable widgets
	variable checkROI
	$widgets(canvas) configure -boxvisible $checkROI
    }

    proc SelectImageSerie { {dir ""} } {
	variable inputFile
	if {$dir eq ""} {
	    set dir [pwd]
	}
	set filename [tk_getOpenFile -initialdir $dir \
			  -title "Choose a file from the image serie"]
	if {$filename ne ""} {
	    # only one file for now
	    set inputFile $filename
	    OnAcceptFile
	}
    }

    proc releaseImages { } {
	variable images
	foreach i [array names images] {
	    image delete $i
	}
	array unset images
    }

    proc createImages { } {
	variable inputSerie
	variable widgets
	set i 0
	foreach f $inputSerie {
	    set images($i) [image create photo -file $f]
	    incr i
	}
	$widgets(canvas) configure -image $images(0)
    }
    
    proc OnAcceptFile { } {
	variable win
	variable inputSerie
	variable inputFile
	# identify series: NOT IMPLEMENTED!

	set inputSerie [list $inputFile]
	releaseImages
	createImages
	$win itemconfigure 0 -state normal
    }

    proc ClearStatus { } {
    }
    
    proc CreateEntryFile {w args} {
	frame $w
	ttk::label $w.lb -text "Input file:"
	set wentry [ttk::entry $w.ent -textvariable "[namespace current]::inputFile"]
	bind $wentry <Return> [namespace code OnAcceptFile]
	button $w.btn -image folder_yellow_16 \
	    -command [namespace code SelectImageSerie]
	grid $w.lb -row 0 -column 0 -sticky "e"
	grid $w.ent -row 0 -column 1 -sticky "ew"
	grid $w.btn -row 0 -column 2 -sticky "w"
	grid columnconfigure $w 1 -weight 1
	return $w
    }

    proc OnClickOK { } {
	variable inputSerie
	variable checkROI
	variable cwd
	variable widgets
	variable lastInput
	global tcl_platform
	
	set result [list -input $inputSerie] 
	if {$checkROI} {
	    foreach {minX minY maxX maxY} [$widgets(canvas) Box GetCoordinates] break
	    set sizeX [expr {$maxX-$minX}]
	    set sizeY [expr {$maxY-$minY}]
	    lappend result -roi [list $minX $minY $sizeX $sizeY]
	}
	set lastInput $result
	OnClose
    }

    proc OnClose { } {
	WM_DELETE_WINDOW
    }
    
    proc WM_DELETE_WINDOW { } {
	variable win
	
	#Dialog::enddialog $win 1
	destroy $win
    }
    
    proc Show { args } {
	variable win
	variable widgets
	variable options
	variable data
	variable checkROI
	variable lastInput
	variable inputFile ""
	
	set w .dlgImport

	set lastInput ""
	set options(-parent) "."
	set checkROI 0
	array set options $args
	if {![winfo exists $w ]} {
	    set wparent $options(-parent)
	    set bg [ $wparent cget -background ]
	    Dialog $w -title [ mc "Import material image" ]  \
		-parent $wparent -modal local -cancel 1 -separator 1 \
		-background $bg -transient false
	    $w add -text [ mc "Import" ] -command [namespace code OnClickOK] \
		-state disabled -background $bg
	    $w add -text [ mc "Close" ]  \
		-background $bg -command [namespace code OnClose]
	    set f [$w getframe]
	    
	    CreateEntryFile $f.inputSerie 
	    grid $f.inputSerie -row 0 -column 0 -sticky snew
	    ttk::labelframe $f.fslice -text [mc "Region of Interest"]
	    grid $f.fslice -row 1 -column 0 -sticky snew -columnspan 2
	    set widgets(checkROI) [ttk::checkbutton $f.fslice.check  -text [mc "Apply ROI"]: \
				       -command [namespace code OnCheckROI] \
				       -variable "[namespace current]::checkROI" ]
	    grid $f.fslice.check -row 0 -column 0 -sticky ne
	    set widgets(canvas) [ClipImage $f.fslice.canvas -background white]
	    #puts "$widgets(canvas) -row 0 -column 0 "
	    set ::f $f.fslice
	    set ::c $widgets(canvas)
	    
	    grid $widgets(canvas) -row 0 -column 1 -sticky snew
	    grid rowconfigure $f.fslice 0 -weight 1
	    grid columnconfigure $f.fslice 1 -weight 1
	    
	    frame $f.fstatus -bd 1 -relief ridge
	    grid $f.fstatus -row 4 -column 0 -sticky snew -columnspan 2
	    grid columnconfigure $f.fstatus 0 -weight 3
	    grid columnconfigure $f.fstatus 1 -weight 1
	    set widgets(status,msg) [ttk::label $f.fstatus.msg]
	    grid $widgets(status,msg) -row 0 -column 0 -sticky snew
	    ProgressBar $f.fstatus.pbar -type infinite -maximum 100 \
		-variable  "[namespace current]::data(progress)"  \
		-bd 1 -relief sunken \
		-troughcolor $bg -foreground #c3c3ff
	    set widgets(status,pbar) $f.fstatus.pbar
	    grid $widgets(status,pbar) -row 0 -column 1 -sticky snew
	    grid columnconfigure $f 0 -weight 1
	    grid rowconfigure $f 1 -weight 1
	    set win $w
	    wm protocol $win WM_DELETE_WINDOW [ namespace code WM_DELETE_WINDOW ]
	} else {
	    
	}
	ClearStatus
    #$widgets(txtTrace) delete 0.0 end
	set f [$win getframe]
	set x [expr [winfo screenwidth $win]/2 - [winfo reqwidth $f]/2 \
		   - [winfo vrootx [winfo parent $win]]]
	set y [expr [winfo screenheight $win]/2 - [winfo reqheight $f]/2 \
		   - [winfo vrooty [winfo parent $win]]]
	$win configure -geometry +$x+$y
	$win draw
	return $lastInput
    }
}

if {0} {

set serie [SelectImageSerie]
set img [image create photo -file [lindex $serie 0]]

ClipImage .ci -image $img -background white
pack .ci -fill both
.ci configure -boxvisible 1

}

if {[ImportImg2Mat::amIMaster]} {

    wm state . withdrawn
    ImportImg2Mat::Show
    
    exit
    #do stuff
}
